package com.rjconsultores.ventaboletos.web.gui.controladores.expressos;

import java.util.Date;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Checkbox;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Paging;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.SolicitudExpreso;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.web.utilerias.MyDatebox;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderDocumentosExpresos;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderExpresosPorCotizar;

@Controller("documentosExpresosController")
@Scope("prototype")
public class DocumentosExpresosController extends MyGenericForwardComposer{
	private static final long serialVersionUID = 1L;
	
	@Autowired
    private transient PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos;
	
	@Autowired
	private EmpresaService empresaService;
	
	private List<Empresa> lsEmpresa;
	private Combobox cmbEmpresa;
	private MyListbox expresosList;
	private Paging pagingExpresos;
	private Checkbox ckServiciosInactivos;
	
	private MyDatebox txtFechaInicio;
	private MyDatebox txtFechaFin;
	
	SolicitudExpreso expreso;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		super.doAfterCompose(comp);
		
		expresosList.setItemRenderer(new RenderDocumentosExpresos(this));
		
		refreshLista();
	}
	
	private void refreshLista() {
		HibernateSearchObject<SolicitudExpreso> buscarExpresos =
                new HibernateSearchObject<SolicitudExpreso>(SolicitudExpreso.class, pagingExpresos.getPageSize());
		
		Date fechaInicio = txtFechaInicio.getValue();
		if(fechaInicio != null) {
			buscarExpresos.addFilterGreaterOrEqual("fechaSolicitud", fechaInicio);
		}
		
		Date fechaFin = txtFechaFin.getValue();
		if(fechaFin != null) {
			buscarExpresos.addFilterLessOrEqual("fechaSolicitud", fechaFin);
		}
		
		if(ckServiciosInactivos.isChecked()) {
			buscarExpresos.addFilterLessOrEqual("ACTIVO", true);
		}
		
		plwTrayectosExpresos.init(buscarExpresos, expresosList, pagingExpresos);
	}
	
	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public Combobox getCmbEmpresa() {
		return cmbEmpresa;
	}

	public void setCmbEmpresa(Combobox cmbEmpresa) {
		this.cmbEmpresa = cmbEmpresa;
	}

	public MyListbox getExpresosList() {
		return expresosList;
	}

	public void setExpresosList(MyListbox expresosList) {
		this.expresosList = expresosList;
	}

	public Paging getPagingExpresos() {
		return pagingExpresos;
	}

	public void setPagingExpresos(Paging pagingExpresos) {
		this.pagingExpresos = pagingExpresos;
	}

	public MyDatebox getTxtFechaInicio() {
		return txtFechaInicio;
	}

	public void setTxtFechaInicio(MyDatebox txtFechaInicio) {
		this.txtFechaInicio = txtFechaInicio;
	}

	public PagedListWrapper<SolicitudExpreso> getPlwTrayectosExpresos() {
		return plwTrayectosExpresos;
	}

	public void setPlwTrayectosExpresos(PagedListWrapper<SolicitudExpreso> plwTrayectosExpresos) {
		this.plwTrayectosExpresos = plwTrayectosExpresos;
	}
}
